#include "stdafx.h"
#include "CppUnitTest.h"

using namespace CryptoPP;
using namespace Microsoft::VisualStudio::CppUnitTestFramework;

namespace CryptoPPTests
{
	TEST_CLASS(SymmetricTestVectorChecks)
	{
	public:
		
		TEST_METHOD(Threefish256TestVectorChecks)
		{
			const word64 InData[4] = {0};
			const word64 Tweak[3] = {0};
			const byte Key[32] = {0};
			word64 IntermediateData[4]={0};
			byte CompareData[32]={0};

			Threefish_256::Encryption Encryptor1(Key);
			Encryptor1.ProcessAndXorBlockWithTweak((byte*)InData,nullptr,(byte*)IntermediateData,(byte*)Tweak,16);
			Assert::IsTrue(IntermediateData[0]==0x94EEEA8B1F2ADA84,L"first word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[1]==0xADF103313EAE6670,L"second word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[2]==0x952419A1F4B16D53,L"third word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[3]==0xD83F13E63C9F6B11,L"fourth word failed (1)",LINE_INFO());
			Threefish_256::Decryption Decryptor1(Key);
			Decryptor1.ProcessAndXorBlockWithTweak((byte*)IntermediateData,nullptr,CompareData,(byte*)Tweak,16);
			Assert::IsTrue(memcmp(CompareData,InData,32)==0,L"out != in",LINE_INFO());

			const word64 NonZeroInData[4] = {0xF8F9FAFBFCFDFEFF,0xF0F1F2F3F4F5F6F7,0xE8E9EAEBECEDEEEF,0xE0E1E2E3E4E5E6E7};
			const word64 NonZeroTweak[3] = {0x0706050403020100,0x0F0E0D0C0B0A0908,0x0808080808080808};
			const word64 NonZeroKey[4] = {0x1716151413121110,0x1F1E1D1C1B1A1918,0x2726252423222120,0x2F2E2D2C2B2A2928};

			Threefish_256::Encryption Encryptor2((byte*)NonZeroKey);
			Encryptor2.ProcessAndXorBlockWithTweak((byte*)NonZeroInData,nullptr,(byte*)IntermediateData,(byte*)NonZeroTweak,16);
			Assert::IsTrue(IntermediateData[0]==0xDF8FEA0EFF91D0E0,L"first word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[1]==0xD50AD82EE69281C9,L"second word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[2]==0x76F48D58085D869D,L"third word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[3]==0xDF975E95B5567065,L"fourth word failed (2)",LINE_INFO());
			Threefish_256::Decryption Decryptor2((byte*)NonZeroKey);
			Decryptor2.ProcessAndXorBlockWithTweak((byte*)IntermediateData,nullptr,CompareData,(byte*)NonZeroTweak,16);
			Assert::IsTrue(memcmp(CompareData,NonZeroInData,32)==0,L"out != in",LINE_INFO());
		}

		TEST_METHOD(Threefish512TestVectorChecks)
		{
			const word64 InData[8] = {0};
			const word64 Tweak[2] = {0};
			const word64 Key[8] = {0};
			word64 IntermediateData[8]={0};
			byte CompareData[64]={0};

			Threefish_512::Encryption Encryptor1((byte*)Key);
			Encryptor1.ProcessAndXorBlockWithTweak((byte*)InData,nullptr,(byte*)IntermediateData,(byte*)Tweak,16);
			Assert::IsTrue(IntermediateData[0]==0xBC2560EFC6BBA2B1,L"first word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[1]==0xE3361F162238EB40,L"second word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[2]==0xFB8631EE0ABBD175,L"third word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[3]==0x7B9479D4C5479ED1,L"fourth word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[4]==0xCFF0356E58F8C27B,L"fifth word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[5]==0xB1B7B08430F0E7F7,L"sixth word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[6]==0xE9A380A56139ABF1,L"seventh word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[7]==0xBE7B6D4AA11EB47E,L"eigth word failed (1)",LINE_INFO());
			           
			Threefish_512::Decryption Decryptor1((byte*)Key);
			Decryptor1.ProcessAndXorBlockWithTweak((byte*)IntermediateData,nullptr,CompareData,(byte*)Tweak,16);
			Assert::IsTrue(memcmp(CompareData,InData,32)==0,L"out != in",LINE_INFO());

			const word64 NonZeroInData[8] = {0xF8F9FAFBFCFDFEFF,0xF0F1F2F3F4F5F6F7,0xE8E9EAEBECEDEEEF,0xE0E1E2E3E4E5E6E7,0xD8D9DADBDCDDDEDF,0xD0D1D2D3D4D5D6D7,0xC8C9CACBCCCDCECF,0xC0C1C2C3C4C5C6C7};
			const word64 NonZeroTweak[2] = {0x0706050403020100,0x0F0E0D0C0B0A0908};
			const word64 NonZeroKey[8] = {0x1716151413121110,0x1F1E1D1C1B1A1918,0x2726252423222120,0x2F2E2D2C2B2A2928,0x3736353433323130,0x3F3E3D3C3B3A3938,0x4746454443424140,0x4F4E4D4C4B4A4948};

			Threefish_512::Encryption Encryptor2((byte*)NonZeroKey);
			Encryptor2.ProcessAndXorBlockWithTweak((byte*)NonZeroInData,nullptr,(byte*)IntermediateData,(byte*)NonZeroTweak,16);
			Assert::IsTrue(IntermediateData[0]==0x2C5AD426964304E3,L"first word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[1]==0x9A2436D6D8CA01B4,L"second word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[2]==0xDD456DB00E333863,L"third word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[3]==0x794725970EB9368B,L"fourth word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[4]==0x043546998D0A2A27,L"fifth word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[5]==0x25A7C918EA204478,L"sixth word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[6]==0x346201A1FEDF11AF,L"seventh word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[7]==0x3DAF1C5C3D672789,L"eigth word failed (2)",LINE_INFO());

			Threefish_512::Decryption Decryptor2((byte*)NonZeroKey);
			Decryptor2.ProcessAndXorBlockWithTweak((byte*)IntermediateData,nullptr,CompareData,(byte*)NonZeroTweak,16);
			Assert::IsTrue(memcmp(CompareData,NonZeroInData,32)==0,L"out != in",LINE_INFO());
		}

		TEST_METHOD(Threefish1024TestVectorChecks)
		{
			const word64 InData[16] = {0};
			const word64 Tweak[2] = {0};
			const word64 Key[16] = {0};
			word64 IntermediateData[16]={0};
			byte CompareData[128]={0};

			Threefish_1024::Encryption Encryptor1((byte*)Key);
			Encryptor1.ProcessAndXorBlockWithTweak((byte*)InData,nullptr,(byte*)IntermediateData,(byte*)Tweak,16);
			Assert::IsTrue(IntermediateData[0]==0x04B3053D0A3D5CF0,L"first word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[1]==0x0136E0D1C7DD85F7,L"second word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[2]==0x067B212F6EA78A5C,L"third word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[3]==0x0DA9C10B4C54E1C6,L"fourth word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[4]==0x0F4EC27394CBACF0,L"fifth word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[5]==0x32437F0568EA4FD5,L"sixth word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[6]==0xCFF56D1D7654B49C,L"seventh word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[7]==0xA2D5FB14369B2E7B,L"eigth word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[8]==0x540306B460472E0B,L"ninth word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[9]==0x71C18254BCEA820D,L"tenth word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[10]==0xC36B4068BEAF32C8,L"eleventh word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[11]==0xFA4329597A360095,L"12. word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[12]==0xC4A36C28434A5B9A,L"13. word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[13]==0xD54331444B1046CF,L"14. word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[14]==0xDF11834830B2A460,L"15. word failed (1)",LINE_INFO());
			Assert::IsTrue(IntermediateData[15]==0x1E39E8DFE1F7EE4F,L"16. word failed (1)",LINE_INFO());

			Threefish_1024::Decryption Decryptor1((byte*)Key);
			Decryptor1.ProcessAndXorBlockWithTweak((byte*)IntermediateData,nullptr,CompareData,(byte*)Tweak,16);
			Assert::IsTrue(memcmp(CompareData,InData,32)==0,L"out != in",LINE_INFO());

			const word64 NonZeroInData[16] = {0xF8F9FAFBFCFDFEFF,0xF0F1F2F3F4F5F6F7,0xE8E9EAEBECEDEEEF,0xE0E1E2E3E4E5E6E7,0xD8D9DADBDCDDDEDF,0xD0D1D2D3D4D5D6D7,0xC8C9CACBCCCDCECF,0xC0C1C2C3C4C5C6C7,
				0xB8B9BABBBCBDBEBF,  0xB0B1B2B3B4B5B6B7,  0xA8A9AAABACADAEAF,  0xA0A1A2A3A4A5A6A7, 
				0x98999A9B9C9D9E9F,  0x9091929394959697,  0x88898A8B8C8D8E8F,  0x8081828384858687};
			const word64 NonZeroTweak[2] = {0x0706050403020100,0x0F0E0D0C0B0A0908};
			const word64 NonZeroKey[16] = {0x1716151413121110,0x1F1E1D1C1B1A1918,0x2726252423222120,0x2F2E2D2C2B2A2928,0x3736353433323130,0x3F3E3D3C3B3A3938,0x4746454443424140,0x4F4E4D4C4B4A4948,
				0x5756555453525150,  0x5F5E5D5C5B5A5958,  0x6766656463626160,  0x6F6E6D6C6B6A6968, 
				0x7776757473727170,  0x7F7E7D7C7B7A7978,  0x8786858483828180,  0x8F8E8D8C8B8A8988};

			Threefish_1024::Encryption Encryptor2((byte*)NonZeroKey);
			Encryptor2.ProcessAndXorBlockWithTweak((byte*)NonZeroInData,nullptr,(byte*)IntermediateData,(byte*)NonZeroTweak,16);
			Assert::IsTrue(IntermediateData[0]==0xB0C33CD7DB4D65A6,L"first word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[1]==0xBC49A85A1077D75D,L"second word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[2]==0x6855FCAFEA7293E4,L"third word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[3]==0x1C5385AB1B7754D2,L"fourth word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[4]==0x30E4AAFFE780F794,L"fifth word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[5]==0xE1BBEE708CAFD8D5,L"sixth word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[6]==0x9CA837B7423B0F76,L"seventh word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[7]==0xBD1403670D4963B3,L"eigth word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[8]==0x451F2E3CE61EA48A,L"ninth word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[9]==0xB360832F9277D4FB,L"tenth word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[10]==0x0AAFC7A65E12D688,L"eleventh word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[11]==0xC8906E79016D05D7,L"12. word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[12]==0xB316570A15F41333,L"13. word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[13]==0x74E98A2869F5D50E,L"14. word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[14]==0x57CE6F9247432BCE,L"15. word failed (2)",LINE_INFO());
			Assert::IsTrue(IntermediateData[15]==0xDE7CDD77215144DE,L"16. word failed (2)",LINE_INFO());

			Threefish_1024::Decryption Decryptor2((byte*)NonZeroKey);
			Decryptor2.ProcessAndXorBlockWithTweak((byte*)IntermediateData,nullptr,CompareData,(byte*)NonZeroTweak,16);
			Assert::IsTrue(memcmp(CompareData,NonZeroInData,32)==0,L"out != in",LINE_INFO());
		}

		TEST_METHOD(Skein256TestVectorChecks)
		{
			Skein Hasher(32,32); // Skein-256-256
			const byte TestData1[] = {0xFF};
			const byte TestVectorResult1[] = {
			0x0B, 0x98, 0xDC, 0xD1, 0x98, 0xEA, 0x0E, 0x50, 0xA7, 0xA2, 0x44, 0xC4, 0x44, 0xE2, 0x5C, 0x23,
			0xDA, 0x30, 0xC1, 0x0F, 0xC9, 0xA1, 0xF2, 0x70, 0xA6, 0x63, 0x7F, 0x1F, 0x34, 0xE6, 0x7E, 0xD2
			};
			Assert::IsTrue(Hasher.VerifyDigest(TestVectorResult1,TestData1,1),L"Skein-256-256 check failed.",LINE_INFO());
			const byte TestData2[] = {
			0xFF, 0xFE, 0xFD, 0xFC, 0xFB, 0xFA, 0xF9, 0xF8, 0xF7, 0xF6, 0xF5, 0xF4, 0xF3, 0xF2, 0xF1, 0xF0,
			0xEF, 0xEE, 0xED, 0xEC, 0xEB, 0xEA, 0xE9, 0xE8, 0xE7, 0xE6, 0xE5, 0xE4, 0xE3, 0xE2, 0xE1, 0xE0};
			const byte TestVectorResult2[] = {
			0x8D, 0x0F, 0xA4, 0xEF, 0x77, 0x7F, 0xD7, 0x59, 0xDF, 0xD4, 0x04, 0x4E, 0x6F, 0x6A, 0x5A, 0xC3,
			0xC7, 0x74, 0xAE, 0xC9, 0x43, 0xDC, 0xFC, 0x07, 0x92, 0x7B, 0x72, 0x3B, 0x5D, 0xBF, 0x40, 0x8B
			};
			Assert::IsTrue(Hasher.VerifyDigest(TestVectorResult2,TestData2,32),L"Skein-256-256 check failed.",LINE_INFO());
			const byte TestData3[] = {
			0xFF, 0xFE, 0xFD, 0xFC, 0xFB, 0xFA, 0xF9, 0xF8, 0xF7, 0xF6, 0xF5, 0xF4, 0xF3, 0xF2, 0xF1, 0xF0,
			0xEF, 0xEE, 0xED, 0xEC, 0xEB, 0xEA, 0xE9, 0xE8, 0xE7, 0xE6, 0xE5, 0xE4, 0xE3, 0xE2, 0xE1, 0xE0,
			0xDF, 0xDE, 0xDD, 0xDC, 0xDB, 0xDA, 0xD9, 0xD8, 0xD7, 0xD6, 0xD5, 0xD4, 0xD3, 0xD2, 0xD1, 0xD0,
			0xCF, 0xCE, 0xCD, 0xCC, 0xCB, 0xCA, 0xC9, 0xC8, 0xC7, 0xC6, 0xC5, 0xC4, 0xC3, 0xC2, 0xC1, 0xC0};
			const byte TestVectorResult3[] = {
			0xDF, 0x28, 0xE9, 0x16, 0x63, 0x0D, 0x0B, 0x44, 0xC4, 0xA8, 0x49, 0xDC, 0x9A, 0x02, 0xF0, 0x7A,
			0x07, 0xCB, 0x30, 0xF7, 0x32, 0x31, 0x82, 0x56, 0xB1, 0x5D, 0x86, 0x5A, 0xC4, 0xAE, 0x16, 0x2F
			};
			Assert::IsTrue(Hasher.VerifyDigest(TestVectorResult3,TestData3,64),L"Skein-256-256 check failed.",LINE_INFO());
		}

		TEST_METHOD(Skein512TestVectorChecks)
		{
			Skein Hasher(64,64); // Skein-512-512
			const byte TestData1[] = {0xFF};
			const byte TestVectorResult1[] = {
			0x71, 0xB7, 0xBC, 0xE6, 0xFE, 0x64, 0x52, 0x22, 0x7B, 0x9C, 0xED, 0x60, 0x14, 0x24, 0x9E, 0x5B,
			0xF9, 0xA9, 0x75, 0x4C, 0x3A, 0xD6, 0x18, 0xCC, 0xC4, 0xE0, 0xAA, 0xE1, 0x6B, 0x31, 0x6C, 0xC8,
			0xCA, 0x69, 0x8D, 0x86, 0x43, 0x07, 0xED, 0x3E, 0x80, 0xB6, 0xEF, 0x15, 0x70, 0x81, 0x2A, 0xC5,
			0x27, 0x2D, 0xC4, 0x09, 0xB5, 0xA0, 0x12, 0xDF, 0x2A, 0x57, 0x91, 0x02, 0xF3, 0x40, 0x61, 0x7A
			};
			Assert::IsTrue(Hasher.VerifyDigest(TestVectorResult1,TestData1,1),L"Skein-512-512 check failed.",LINE_INFO());
			const byte TestData2[] = {
			0xFF, 0xFE, 0xFD, 0xFC, 0xFB, 0xFA, 0xF9, 0xF8, 0xF7, 0xF6, 0xF5, 0xF4, 0xF3, 0xF2, 0xF1, 0xF0,
			0xEF, 0xEE, 0xED, 0xEC, 0xEB, 0xEA, 0xE9, 0xE8, 0xE7, 0xE6, 0xE5, 0xE4, 0xE3, 0xE2, 0xE1, 0xE0,
			0xDF, 0xDE, 0xDD, 0xDC, 0xDB, 0xDA, 0xD9, 0xD8, 0xD7, 0xD6, 0xD5, 0xD4, 0xD3, 0xD2, 0xD1, 0xD0,
			0xCF, 0xCE, 0xCD, 0xCC, 0xCB, 0xCA, 0xC9, 0xC8, 0xC7, 0xC6, 0xC5, 0xC4, 0xC3, 0xC2, 0xC1, 0xC0};
			const byte TestVectorResult2[] = {
			0x45, 0x86, 0x3B, 0xA3, 0xBE, 0x0C, 0x4D, 0xFC, 0x27, 0xE7, 0x5D, 0x35, 0x84, 0x96, 0xF4, 0xAC,
			0x9A, 0x73, 0x6A, 0x50, 0x5D, 0x93, 0x13, 0xB4, 0x2B, 0x2F, 0x5E, 0xAD, 0xA7, 0x9F, 0xC1, 0x7F,
			0x63, 0x86, 0x1E, 0x94, 0x7A, 0xFB, 0x1D, 0x05, 0x6A, 0xA1, 0x99, 0x57, 0x5A, 0xD3, 0xF8, 0xC9,
			0xA3, 0xCC, 0x17, 0x80, 0xB5, 0xE5, 0xFA, 0x4C, 0xAE, 0x05, 0x0E, 0x98, 0x98, 0x76, 0x62, 0x5B
			};
			Assert::IsTrue(Hasher.VerifyDigest(TestVectorResult2,TestData2,64),L"Skein-512-512 check failed.",LINE_INFO());
			const byte TestData3[] = {
			0xFF, 0xFE, 0xFD, 0xFC, 0xFB, 0xFA, 0xF9, 0xF8, 0xF7, 0xF6, 0xF5, 0xF4, 0xF3, 0xF2, 0xF1, 0xF0,
			0xEF, 0xEE, 0xED, 0xEC, 0xEB, 0xEA, 0xE9, 0xE8, 0xE7, 0xE6, 0xE5, 0xE4, 0xE3, 0xE2, 0xE1, 0xE0,
			0xDF, 0xDE, 0xDD, 0xDC, 0xDB, 0xDA, 0xD9, 0xD8, 0xD7, 0xD6, 0xD5, 0xD4, 0xD3, 0xD2, 0xD1, 0xD0,
			0xCF, 0xCE, 0xCD, 0xCC, 0xCB, 0xCA, 0xC9, 0xC8, 0xC7, 0xC6, 0xC5, 0xC4, 0xC3, 0xC2, 0xC1, 0xC0,
			0xBF, 0xBE, 0xBD, 0xBC, 0xBB, 0xBA, 0xB9, 0xB8, 0xB7, 0xB6, 0xB5, 0xB4, 0xB3, 0xB2, 0xB1, 0xB0,
			0xAF, 0xAE, 0xAD, 0xAC, 0xAB, 0xAA, 0xA9, 0xA8, 0xA7, 0xA6, 0xA5, 0xA4, 0xA3, 0xA2, 0xA1, 0xA0,
			0x9F, 0x9E, 0x9D, 0x9C, 0x9B, 0x9A, 0x99, 0x98, 0x97, 0x96, 0x95, 0x94, 0x93, 0x92, 0x91, 0x90,
			0x8F, 0x8E, 0x8D, 0x8C, 0x8B, 0x8A, 0x89, 0x88, 0x87, 0x86, 0x85, 0x84, 0x83, 0x82, 0x81, 0x80};
			const byte TestVectorResult3[] = {
			0x91, 0xCC, 0xA5, 0x10, 0xC2, 0x63, 0xC4, 0xDD, 0xD0, 0x10, 0x53, 0x0A, 0x33, 0x07, 0x33, 0x09,
			0x62, 0x86, 0x31, 0xF3, 0x08, 0x74, 0x7E, 0x1B, 0xCB, 0xAA, 0x90, 0xE4, 0x51, 0xCA, 0xB9, 0x2E,
			0x51, 0x88, 0x08, 0x7A, 0xF4, 0x18, 0x87, 0x73, 0xA3, 0x32, 0x30, 0x3E, 0x66, 0x67, 0xA7, 0xA2,
			0x10, 0x85, 0x6F, 0x74, 0x21, 0x39, 0x00, 0x00, 0x71, 0xF4, 0x8E, 0x8B, 0xA2, 0xA5, 0xAD, 0xB7
			};
			Assert::IsTrue(Hasher.VerifyDigest(TestVectorResult3,TestData3,128),L"Skein-512-512 check failed.",LINE_INFO());
		}

		TEST_METHOD(Skein1024TestVectorChecks)
		{
			FixedSizeSecBlock<byte,256> TestData;
			memset_z(TestData,0,256);
			TestData[0]=0xFF;

			Skein Hasher(128,128); // Skein-1024-1024
			const byte TestVectorResult[] = {
			0xE6, 0x2C, 0x05, 0x80, 0x2E, 0xA0, 0x15, 0x24, 0x07, 0xCD, 0xD8, 0x78, 0x7F, 0xDA, 0x9E, 0x35,
			0x70, 0x3D, 0xE8, 0x62, 0xA4, 0xFB, 0xC1, 0x19, 0xCF, 0xF8, 0x59, 0x0A, 0xFE, 0x79, 0x25, 0x0B,
			0xCC, 0xC8, 0xB3, 0xFA, 0xF1, 0xBD, 0x24, 0x22, 0xAB, 0x5C, 0x0D, 0x26, 0x3F, 0xB2, 0xF8, 0xAF,
			0xB3, 0xF7, 0x96, 0xF0, 0x48, 0x00, 0x03, 0x81, 0x53, 0x1B, 0x6F, 0x00, 0xD8, 0x51, 0x61, 0xBC,
			0x0F, 0xFF, 0x4B, 0xEF, 0x24, 0x86, 0xB1, 0xEB, 0xCD, 0x37, 0x73, 0xFA, 0xBF, 0x50, 0xAD, 0x4A,
			0xD5, 0x63, 0x9A, 0xF9, 0x04, 0x0E, 0x3F, 0x29, 0xC6, 0xC9, 0x31, 0x30, 0x1B, 0xF7, 0x98, 0x32,
			0xE9, 0xDA, 0x09, 0x85, 0x7E, 0x83, 0x1E, 0x82, 0xEF, 0x8B, 0x46, 0x91, 0xC2, 0x35, 0x65, 0x65,
			0x15, 0xD4, 0x37, 0xD2, 0xBD, 0xA3, 0x3B, 0xCE, 0xC0, 0x01, 0xC6, 0x7F, 0xFD, 0xE1, 0x5B, 0xA8};
			Assert::IsTrue(Hasher.VerifyDigest(TestVectorResult,TestData,1),L"Skein-1024-1024 check failed.",LINE_INFO());

			const byte TestData2 [] = {
			0xFF, 0xFE, 0xFD, 0xFC, 0xFB, 0xFA, 0xF9, 0xF8, 0xF7, 0xF6, 0xF5, 0xF4, 0xF3, 0xF2, 0xF1, 0xF0,
			0xEF, 0xEE, 0xED, 0xEC, 0xEB, 0xEA, 0xE9, 0xE8, 0xE7, 0xE6, 0xE5, 0xE4, 0xE3, 0xE2, 0xE1, 0xE0,
			0xDF, 0xDE, 0xDD, 0xDC, 0xDB, 0xDA, 0xD9, 0xD8, 0xD7, 0xD6, 0xD5, 0xD4, 0xD3, 0xD2, 0xD1, 0xD0,
			0xCF, 0xCE, 0xCD, 0xCC, 0xCB, 0xCA, 0xC9, 0xC8, 0xC7, 0xC6, 0xC5, 0xC4, 0xC3, 0xC2, 0xC1, 0xC0,
			0xBF, 0xBE, 0xBD, 0xBC, 0xBB, 0xBA, 0xB9, 0xB8, 0xB7, 0xB6, 0xB5, 0xB4, 0xB3, 0xB2, 0xB1, 0xB0,
			0xAF, 0xAE, 0xAD, 0xAC, 0xAB, 0xAA, 0xA9, 0xA8, 0xA7, 0xA6, 0xA5, 0xA4, 0xA3, 0xA2, 0xA1, 0xA0,
			0x9F, 0x9E, 0x9D, 0x9C, 0x9B, 0x9A, 0x99, 0x98, 0x97, 0x96, 0x95, 0x94, 0x93, 0x92, 0x91, 0x90,
			0x8F, 0x8E, 0x8D, 0x8C, 0x8B, 0x8A, 0x89, 0x88, 0x87, 0x86, 0x85, 0x84, 0x83, 0x82, 0x81, 0x80
			};

			const byte TestVectorResult2[] = {
			0x1F, 0x3E, 0x02, 0xC4, 0x6F, 0xB8, 0x0A, 0x3F, 0xCD, 0x2D, 0xFB, 0xBC, 0x7C, 0x17, 0x38, 0x00,
			0xB4, 0x0C, 0x60, 0xC2, 0x35, 0x4A, 0xF5, 0x51, 0x18, 0x9E, 0xBF, 0x43, 0x3C, 0x3D, 0x85, 0xF9,
			0xFF, 0x18, 0x03, 0xE6, 0xD9, 0x20, 0x49, 0x31, 0x79, 0xED, 0x7A, 0xE7, 0xFC, 0xE6, 0x9C, 0x35,
			0x81, 0xA5, 0xA2, 0xF8, 0x2D, 0x3E, 0x0C, 0x7A, 0x29, 0x55, 0x74, 0xD0, 0xCD, 0x7D, 0x21, 0x7C,
			0x48, 0x4D, 0x2F, 0x63, 0x13, 0xD5, 0x9A, 0x77, 0x18, 0xEA, 0xD0, 0x7D, 0x07, 0x29, 0xC2, 0x48,
			0x51, 0xD7, 0xE7, 0xD2, 0x49, 0x1B, 0x90, 0x2D, 0x48, 0x91, 0x94, 0xE6, 0xB7, 0xD3, 0x69, 0xDB,
			0x0A, 0xB7, 0xAA, 0x10, 0x6F, 0x0E, 0xE0, 0xA3, 0x9A, 0x42, 0xEF, 0xC5, 0x4F, 0x18, 0xD9, 0x37,
			0x76, 0x08, 0x09, 0x85, 0xF9, 0x07, 0x57, 0x4F, 0x99, 0x5E, 0xC6, 0xA3, 0x71, 0x53, 0xA5, 0x78};
			Assert::IsTrue(Hasher.VerifyDigest(TestVectorResult2,TestData2,128),L"Skein-1024-1024 check failed.",LINE_INFO());

			const byte TestData3 [] = {
			0xFF, 0xFE, 0xFD, 0xFC, 0xFB, 0xFA, 0xF9, 0xF8, 0xF7, 0xF6, 0xF5, 0xF4, 0xF3, 0xF2, 0xF1, 0xF0,
			0xEF, 0xEE, 0xED, 0xEC, 0xEB, 0xEA, 0xE9, 0xE8, 0xE7, 0xE6, 0xE5, 0xE4, 0xE3, 0xE2, 0xE1, 0xE0,
			0xDF, 0xDE, 0xDD, 0xDC, 0xDB, 0xDA, 0xD9, 0xD8, 0xD7, 0xD6, 0xD5, 0xD4, 0xD3, 0xD2, 0xD1, 0xD0,
			0xCF, 0xCE, 0xCD, 0xCC, 0xCB, 0xCA, 0xC9, 0xC8, 0xC7, 0xC6, 0xC5, 0xC4, 0xC3, 0xC2, 0xC1, 0xC0,
			0xBF, 0xBE, 0xBD, 0xBC, 0xBB, 0xBA, 0xB9, 0xB8, 0xB7, 0xB6, 0xB5, 0xB4, 0xB3, 0xB2, 0xB1, 0xB0,
			0xAF, 0xAE, 0xAD, 0xAC, 0xAB, 0xAA, 0xA9, 0xA8, 0xA7, 0xA6, 0xA5, 0xA4, 0xA3, 0xA2, 0xA1, 0xA0,
			0x9F, 0x9E, 0x9D, 0x9C, 0x9B, 0x9A, 0x99, 0x98, 0x97, 0x96, 0x95, 0x94, 0x93, 0x92, 0x91, 0x90,
			0x8F, 0x8E, 0x8D, 0x8C, 0x8B, 0x8A, 0x89, 0x88, 0x87, 0x86, 0x85, 0x84, 0x83, 0x82, 0x81, 0x80,
			0x7F, 0x7E, 0x7D, 0x7C, 0x7B, 0x7A, 0x79, 0x78, 0x77, 0x76, 0x75, 0x74, 0x73, 0x72, 0x71, 0x70,
			0x6F, 0x6E, 0x6D, 0x6C, 0x6B, 0x6A, 0x69, 0x68, 0x67, 0x66, 0x65, 0x64, 0x63, 0x62, 0x61, 0x60,
			0x5F, 0x5E, 0x5D, 0x5C, 0x5B, 0x5A, 0x59, 0x58, 0x57, 0x56, 0x55, 0x54, 0x53, 0x52, 0x51, 0x50,
			0x4F, 0x4E, 0x4D, 0x4C, 0x4B, 0x4A, 0x49, 0x48, 0x47, 0x46, 0x45, 0x44, 0x43, 0x42, 0x41, 0x40,
			0x3F, 0x3E, 0x3D, 0x3C, 0x3B, 0x3A, 0x39, 0x38, 0x37, 0x36, 0x35, 0x34, 0x33, 0x32, 0x31, 0x30,
			0x2F, 0x2E, 0x2D, 0x2C, 0x2B, 0x2A, 0x29, 0x28, 0x27, 0x26, 0x25, 0x24, 0x23, 0x22, 0x21, 0x20,
			0x1F, 0x1E, 0x1D, 0x1C, 0x1B, 0x1A, 0x19, 0x18, 0x17, 0x16, 0x15, 0x14, 0x13, 0x12, 0x11, 0x10,
			0x0F, 0x0E, 0x0D, 0x0C, 0x0B, 0x0A, 0x09, 0x08, 0x07, 0x06, 0x05, 0x04, 0x03, 0x02, 0x01, 0x00
			};

			const byte TestVectorResult3[] = {
			0x84, 0x2A, 0x53, 0xC9, 0x9C, 0x12, 0xB0, 0xCF, 0x80, 0xCF, 0x69, 0x49, 0x1B, 0xE5, 0xE2, 0xF7,
			0x51, 0x5D, 0xE8, 0x73, 0x3B, 0x6E, 0xA9, 0x42, 0x2D, 0xFD, 0x67, 0x66, 0x65, 0xB5, 0xFA, 0x42,
			0xFF, 0xB3, 0xA9, 0xC4, 0x8C, 0x21, 0x77, 0x77, 0x95, 0x08, 0x48, 0xCE, 0xCD, 0xB4, 0x8F, 0x64,
			0x0F, 0x81, 0xFB, 0x92, 0xBE, 0xF6, 0xF8, 0x8F, 0x7A, 0x85, 0xC1, 0xF7, 0xCD, 0x14, 0x46, 0xC9,
			0x16, 0x1C, 0x0A, 0xFE, 0x8F, 0x25, 0xAE, 0x44, 0x4F, 0x40, 0xD3, 0x68, 0x00, 0x81, 0xC3, 0x5A,
			0xA4, 0x3F, 0x64, 0x0F, 0xD5, 0xFA, 0x3C, 0x3C, 0x03, 0x0B, 0xCC, 0x06, 0xAB, 0xAC, 0x01, 0xD0,
			0x98, 0xBC, 0xC9, 0x84, 0xEB, 0xD8, 0x32, 0x27, 0x12, 0x92, 0x1E, 0x00, 0xB1, 0xBA, 0x07, 0xD6,
			0xD0, 0x1F, 0x26, 0x90, 0x70, 0x50, 0x25, 0x5E, 0xF2, 0xC8, 0xE2, 0x4F, 0x71, 0x6C, 0x52, 0xA5};
			Assert::IsTrue(Hasher.VerifyDigest(TestVectorResult3,TestData3,256),L"Skein-1024-1024 check failed.",LINE_INFO());
		}

	};
}